// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "driver.h"

ModemConfig::ModemConfig() :
Keydata("/bayonne/vmodem")
{
	static KEYDEF defkeys[] = {
	{"devices", "ttyS1"},
	{"initial", "ATZ"},
	{"speed", "115200"},
	{NULL, NULL}};

	if(chkHosting(NULL))
		Load("/hosting/vmodem");

	Load(defkeys);
}

unsigned ModemConfig::getStack(void)
{
	const char *cp = getLast("stack");

	if(!cp)
		return keythreads.getStack();

	return atoi(cp);
}

ModemDriver::ModemDriver() :
Driver()
{
	port_count = 0;
	char buffer[256];
	char devname[65];
	char *cp;
	char lockname[65];

	groups = new TrunkGroup *[MAX_CAPACITY];
	strcpy(buffer, modemivr.getDevices());
	cp = strtok_r(buffer, " \t\n:,", &cp);
	while(cp)
	{
		if(*cp == '/')
			devname[0] = 0;
		else
			strcpy(devname, "/dev/");
		strcat(devname, cp);

		cp = strrchr(devname, '/');
		++cp;

		cp = strtok_r(NULL, " \t\n:,", &cp);
	}

	if(ports)
		memset(ports, 0, sizeof(ModemTrunk *) * port_count);

	if(groups)
		memset(groups, 0, sizeof(TrunkGroup *) * port_count);

	slog(SLOG_INFO) << "Voice modem driver loaded; capacity=" << port_count << endl;
}

ModemDriver::~ModemDriver()
{
	Stop();

	if(groups)
		delete[] groups;
}

int ModemDriver::Start(void)
{
	int count = 0;
	int port = 0;

	if(active)
	{
		slog(SLOG_ERROR) << "driver already started" << endl;
		return 0;
	}

	slog(SLOG_INFO) << "driver starting..." << endl;
	for(port = 0; port < port_count; ++port)
		ports[port]->Start();

	active = true;
	return count;
}

void ModemDriver::Stop(void)
{
	int port;

	if(!active)
		return;

	for(port = 0; port < port_count; ++port)
		ports[port]->Terminate();


	if(ports)
		memset(ports, 0, sizeof(ModemTrunk *) * port_count);

	active = false;
	slog(SLOG_INFO) << "driver stopping..." << endl;
}

Trunk *ModemDriver::getTrunkPort(int id)
{
	if(id < 0 || id >= port_count)
		return NULL;

	if(!ports)
		return NULL;

	return (Trunk *)ports[id];
}

ModemDriver modemivr;
